#define MICO_CONF_IMR
#include <CORBA.h>
#include <iostream.h>
#include "hello.h"
#include <mico/CosTrading.h>
#include <mico/CosTradingRepos.h>
#include <mico/GenericFactory_impl.h>

#ifdef USE_CORBA_2_1
#define CORBA_CXX_PREFIX(x) _##x
#else
#define CORBA_CXX_PREFIX(x) _cxx_##x
#endif


class Hello_impl : virtual public POA_Hello,
		   virtual public POA_CosLifeCycle::LifeCycleObject
{
    char * hello_string_;
public:
    Hello_impl() { hello_string_ = (char*)NULL; };
    Hello_impl(const char * str) { hello_string_ = CORBA::string_dup(str); };
    virtual char* hello_string() { return CORBA::string_dup(hello_string_); };
    virtual void hello_string( const char* value )
	{
	    CORBA::string_free(hello_string_);
	    hello_string_ = CORBA::string_dup(value);
	};
    
    virtual CosLifeCycle::LifeCycleObject_ptr
	    copy( CosLifeCycle::FactoryFinder_ptr there, const CosLifeCycle::Criteria& the_criteria );
    virtual void move( CosLifeCycle::FactoryFinder_ptr there, const CosLifeCycle::Criteria& the_criteria );
    virtual void remove();
    char * hello (const char *s)
    {
	if (hello_string_)
	    cout << hello_string_<<' ';
	cout << s << endl;
	return CORBA::string_dup (s);
    }
};

CosLifeCycle::LifeCycleObject_ptr
Hello_impl::copy(CosLifeCycle::FactoryFinder_ptr there, const CosLifeCycle::Criteria& the_criteria )
{
    CosLifeCycle::Key factory_id_;
    factory_id_.length(4);
    factory_id_[0].id = CORBA::string_dup("::CosLifeCycle::LifeCycleObject");
    factory_id_[0].kind = CORBA::string_dup("object interface");
    factory_id_[1].id = CORBA::string_dup("");
    factory_id_[1].kind = CORBA::string_dup("implementation equivalence class");
    factory_id_[2].id = CORBA::string_dup("::Hello");
    factory_id_[2].kind = CORBA::string_dup("object implementation");
    factory_id_[3].id = CORBA::string_dup("GenericFactory");
    factory_id_[3].kind = CORBA::string_dup("factory interface");
    CosLifeCycle::Factories* object_factories_ptr = there -> find_factories(factory_id_);
    if (!(*object_factories_ptr).length())
    {
	if (object_factories_ptr) delete object_factories_ptr;
	CosLifeCycle::NoFactory ex;
	ex.search_key = factory_id_;
	mico_throw(ex);
    }

    CORBA::ULong i;
    CosLifeCycle::GenericFactory_ptr hellofactory_ptr;
    Hello_ptr new_obj;
    for (i = 0;i < (*object_factories_ptr).length();++i)
    {
	hellofactory_ptr = CosLifeCycle::GenericFactory::_narrow((*object_factories_ptr)[i]);
	if (CORBA::is_nil(hellofactory_ptr))
	    continue;
	new_obj = Hello::_narrow(hellofactory_ptr->create_object(factory_id_,the_criteria));
	if (CORBA::is_nil(new_obj))
	    continue;
	break;
    }
    
    if (i == (*object_factories_ptr).length())
    {
	if (object_factories_ptr) delete object_factories_ptr;
	CosLifeCycle::NoFactory ex;
	ex.search_key = factory_id_;
	mico_throw(ex);
    }
    delete object_factories_ptr;
    return new_obj;
}

void
Hello_impl::move(CosLifeCycle::FactoryFinder_ptr there, const CosLifeCycle::Criteria& the_criteria )
{
};

void
Hello_impl::remove()
{
#if USE_VAR
    PortableServer::ObjectId_var oid = _default_POA ()->servant_to_id (this);
#else
    PortableServer::ObjectId* oid = _default_POA ()->servant_to_id (this);
#endif
    _default_POA ()->deactivate_object (*oid);

#ifndef USE_VAR
    delete oid;
#endif
    delete this;
}

class HelloFactory_impl : virtual public GenericFactory_impl
{
public:
    CORBA::Object_ptr create_object (const CosLifeCycle::Key &factory_key,
				     const CosLifeCycle::Criteria &the_criteria)
    {
	char * str = (char *)NULL;
	CosLifeCycle::Criteria init_param;
	init_param.length(0);
	for(CORBA::ULong i = 0;i < the_criteria.length();++i)
	    if (!strcmp(the_criteria[i].name,"initialization"))
	    {
		the_criteria[i].value >>= init_param;
		break;
	    }
	Hello_impl* hello_impl = NULL;
	if (!init_param.length())
	    hello_impl = new Hello_impl;
	else
	{
	    for(CORBA::ULong i = 0;i < init_param.length();++i)
		if (!strcmp(init_param[i].name,"hello string"))
		{
		    init_param[i].value >>= str;
		    break;
		}
	    if (str)
		hello_impl = new Hello_impl(CORBA::string_dup(str));
	    else
		hello_impl = new Hello_impl;
	}
	Hello_ptr hello = hello_impl->_this();
//	CORBA::string_free(str);
	return Hello::_duplicate(hello);
    };
};

int
main (int argc, char *argv[])
{
    CORBA::ORB_var orb = CORBA::ORB_init (argc, argv, "mico-local-orb");
    CORBA::Object_var poaobj = orb->resolve_initial_references ("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow (poaobj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();

    HelloFactory_impl* hello = new HelloFactory_impl();
    poa->activate_object (hello);

    CORBA::Object_var tr_obj;
#ifdef HAVE_EXCEPTIONS
    try {
#endif
	tr_obj = orb -> resolve_initial_references("TradingService");
#ifdef HAVE_EXCEPTIONS
    } catch(CORBA::ORB::InvalidName&) {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
#endif
    if (CORBA::is_nil(tr_obj))
    {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
    CosTrading::Lookup_var trader_ = CosTrading::Lookup::_narrow (tr_obj);
    CosTrading::Register_var reg = trader_->register_if();
    if (CORBA::is_nil(reg))
    {
	cerr << "Invalid Trading Service. Abort" << endl;
	exit(1);
    }
    CORBA::Object_var obj = trader_->type_repos();
    CosTradingRepos::ServiceTypeRepository_var repo =
	CosTradingRepos::ServiceTypeRepository::_narrow(obj);
    if (CORBA::is_nil(repo))
    {
	cerr << "Invalid Trading Service. Abort" << endl;
	exit(1);
    }

    CORBA::Boolean gen_type = false;
    CosTradingRepos::ServiceTypeRepository::SpecifiedServiceTypes all_;
    all_._d(CosTradingRepos::ServiceTypeRepository::all);
    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq* list_of_type = 
	repo -> list_types(all_);
    for( CORBA::ULong i = 0;i < list_of_type->length();++i)
    {
	CosTradingRepos::ServiceTypeRepository::TypeStruct* tmp_type =
		repo -> describe_type((*list_of_type)[i]);
	if (!strcmp("GenericFactory",(*list_of_type)[i]) &&
	    !strcmp("IDL:omg.org/CosLifeCycle/GenericFactory:1.0",tmp_type->if_name))
	{
	    delete tmp_type;
	    break;
	}
    }
    delete list_of_type;
    if (!gen_type)
    {
	CosTradingRepos::ServiceTypeRepository::IncarnationNumber i1;
	{
	    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq super;
	    super.length( 0 );
	    CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
	    props.length( 3 );
	    props[0].name = CORBA::string_dup( "ObjectInterface" );
	    props[0].value_type = CORBA::_tc_string;
	    props[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
	    props[1].name = CORBA::string_dup( "ImplEquivClass" );
	    props[1].value_type = CORBA::_tc_string;
	    props[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
	    props[2].name = CORBA::string_dup( "ObjectImpl" );
	    props[2].value_type = CORBA::_tc_string;
	    props[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

	    i1 = repo->add_type( "GenericFactory", "IDL:omg.org/CosLifeCycle/GenericFactory:1.0", props, super );
	}
    }
    CosTrading::OfferId id1;

    CosTrading::PropertySeq seq;
    seq.length(3);
    seq[0].name = CORBA::string_dup( "ObjectInterface" );
    seq[0].value <<= CORBA::Any::from_string( "::CosLifeCycle::LifeCycleObject", 0 );
    seq[1].name = CORBA::string_dup( "ImplEquivClass" );
    seq[1].value <<= CORBA::Any::from_string("", 0);
    seq[2].name = CORBA::string_dup( "ObjectImpl" );
    seq[2].value <<= CORBA::Any::from_string("::Hello", 0);

    id1 = reg->CORBA_CXX_PREFIX(export)(hello->_this(),"GenericFactory", seq );

    mgr->activate ();

    orb->run ();
    return 0;
}
