/*
 *  Demo of COSS Relationship Service
 *  Copyright (C) 1998 Karel Gardas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 *  or to my private e-mail:
 *                 gardask@alpha.inf.upol.cz
 */


#include <iostream.h>
#include <fstream.h>
#include <unistd.h>
#include "Documents.h"
#include "Documents_impl.h"
#include <mico/CosTrading.h>
#include <mico/CosTradingRepos.h>

#ifdef USE_CORBA_2_1
#define CORBA_CXX_PREFIX(x) _##x
#else
#define CORBA_CXX_PREFIX(x) _cxx_##x
#endif


int main( int argc, char *argv[] )
{
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );

    CORBA::Object_var poaobj = orb->resolve_initial_references ("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow (poaobj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();

    FigureFactory_impl* ff = new FigureFactory_impl;

    TextFactory_impl* tf = new TextFactory_impl;

    poa->activate_object (ff);
    poa->activate_object (tf);

    CORBA::Object_var tr_obj;
#ifdef HAVE_EXCEPTIONS
    try {
#endif
	tr_obj = orb -> resolve_initial_references("TradingService");
#ifdef HAVE_EXCEPTIONS
    } catch(CORBA::ORB::InvalidName&) {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
#endif
    if (CORBA::is_nil(tr_obj))
    {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
    CosTrading::Lookup_var trader_ = CosTrading::Lookup::_narrow (tr_obj);
    CosTrading::Register_var reg = trader_->register_if();
    if (CORBA::is_nil(reg))
    {
	cerr << "Invalid Trading Service. Abort" << endl;
	exit(1);
    }
    CORBA::Object_var obj = trader_->type_repos();
    CosTradingRepos::ServiceTypeRepository_var repo =
	CosTradingRepos::ServiceTypeRepository::_narrow(obj);
    if (CORBA::is_nil(repo))
    {
	cerr << "Invalid Trading Service. Abort" << endl;
	exit(1);
    }

    CORBA::Boolean fig_type = false;
    CORBA::Boolean text_type = false;
    CosTradingRepos::ServiceTypeRepository::SpecifiedServiceTypes all_;
    all_._d(CosTradingRepos::ServiceTypeRepository::all);
    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq* list_of_type = 
	repo -> list_types(all_);
    for( CORBA::ULong i = 0;i < list_of_type->length();++i)
    {
	CosTradingRepos::ServiceTypeRepository::TypeStruct* tmp_type =
		repo -> describe_type((*list_of_type)[i]);
	if (!strcmp("TextFactory",(*list_of_type)[i]) &&
	    !strcmp("IDL:omg.org/Documents/TextFactory:1.0",tmp_type->if_name))
	    text_type = true;
	if (!strcmp("FigureFactory",(*list_of_type)[i]) &&
	    !strcmp("IDL:omg.org/Documents/FigureFactory:1.0",tmp_type->if_name))
	    fig_type = true;
	delete tmp_type;
	if (text_type && fig_type)
	    break;
    }
    delete list_of_type;
    if (!text_type)
    {
	CosTradingRepos::ServiceTypeRepository::IncarnationNumber i1;
	{
	    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq super;
	    super.length( 0 );
	    CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
	    props.length( 3 );
	    props[0].name = CORBA::string_dup( "ObjectInterface" );
	    props[0].value_type = CORBA::_tc_string;
	    props[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
	    props[1].name = CORBA::string_dup( "ImplEquivClass" );
	    props[1].value_type = CORBA::_tc_string;
	    props[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
	    props[2].name = CORBA::string_dup( "ObjectImpl" );
	    props[2].value_type = CORBA::_tc_string;
	    props[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

	    i1 = repo->add_type( "TextFactory", "IDL:omg.org/Documents/TextFactory:1.0", props, super );
	}
    }
    if (!fig_type)
    {
	CosTradingRepos::ServiceTypeRepository::IncarnationNumber i2;
	{
	    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq super;
	    super.length( 0 );
	    CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
	    props.length( 3 );
	    props[0].name = CORBA::string_dup( "ObjectInterface" );
	    props[0].value_type = CORBA::_tc_string;
	    props[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
	    props[1].name = CORBA::string_dup( "ImplEquivClass" );
	    props[1].value_type = CORBA::_tc_string;
	    props[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
	    props[2].name = CORBA::string_dup( "ObjectImpl" );
	    props[2].value_type = CORBA::_tc_string;
	    props[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
    
	    i2 = repo->add_type( "FigureFactory", "IDL:omg.org/Documents/FigureFactory:1.0", props, super );
    	    }
    }
    CosTrading::OfferId id1;
    CosTrading::OfferId id2;

    CosTrading::PropertySeq seq;
    seq.length(3);
    seq[0].name = CORBA::string_dup( "ObjectInterface" );
    seq[0].value <<= CORBA::Any::from_string( "::Documents::Text", 0 );
    seq[1].name = CORBA::string_dup( "ImplEquivClass" );
    seq[1].value <<= CORBA::Any::from_string("", 0);
    seq[2].name = CORBA::string_dup( "ObjectImpl" );
    seq[2].value <<= CORBA::Any::from_string("", 0);

    id1 = reg->CORBA_CXX_PREFIX(export)(tf->_this(),
					    "TextFactory", seq );

    seq.length(3);
    seq[0].name = CORBA::string_dup( "ObjectInterface" );
    seq[0].value <<= CORBA::Any::from_string( "::Documents::Figure", 0 );
    seq[1].name = CORBA::string_dup( "ImplEquivClass" );
    seq[1].value <<= CORBA::Any::from_string("", 0);
    seq[2].name = CORBA::string_dup( "ObjectImpl" );
    seq[2].value <<= CORBA::Any::from_string("", 0);

    id2 = reg->CORBA_CXX_PREFIX(export)(ff->_this(),
					    "FigureFactory", seq );

    mgr->activate ();

    orb->run ();
#if DEBUG
    cout << "  <FigureServer> END!\n";
#endif
    return 0;
}





